using System;
using System.Collections;
using setPoint.weaving;
using setPoint.messageReifying;

namespace LENDLConfiguration {
	/// <summary>
	/// Avoids recalculating results of the message by caching them into a Hashtable
	/// </summary>
	public class CachingAspect : IAspect {

		private Hashtable _cache = new Hashtable();
		private bool _cacheHit;

		public CachingAspect() {
		}

		public void lookUp(setPoint.messageReifying.IJoinPoint aJoinpoint) {
			HashtableArrayKey key = this.keyForJoinPoint(aJoinpoint);
			if(this._cacheHit = this._cache.Contains(key))
				aJoinpoint.returnValue = this._cache[key];		
		}	

		public void afterLookUp(setPoint.messageReifying.IJoinPoint aJoinpoint) {
			if(!this._cacheHit)
				this._cache.Add(this.keyForJoinPoint(aJoinpoint),aJoinpoint.returnValue);
		}

		public void flush(setPoint.messageReifying.IJoinPoint aJoinpoint) 
		{
			this._cache = new Hashtable();
			this._cacheHit = false;
		}

		public bool mustCancelExecution {
			get { return this._cacheHit; }
		}	

		private HashtableArrayKey keyForJoinPoint(IJoinPoint aJoinpoint) {
			ArrayList key = new ArrayList();
			key.Add(aJoinpoint.message.uri);
			foreach(object argument in aJoinpoint.arguments)
				key.Add(argument);
			return new HashtableArrayKey(key);
		}
		internal class HashtableArrayKey {
			private IList _key;

			internal HashtableArrayKey(IList key) {
				this._key = key;	
			}

			public override int GetHashCode() {
				int result=0;				
				foreach(object element in this._key)
					result += element.GetHashCode();				
				return result;
			}
		 
			public override bool Equals(Object obj) {
				if(!(obj is HashtableArrayKey))
					return false;
				bool result=true;
				HashtableArrayKey cmp = obj as HashtableArrayKey;
				for(int i=0;i<this._key.Count;i++)
					result = result && this._key[i].Equals(cmp._key[i]);
				return result;
			}
		}
	}
}