using System;
using System.Collections;
using dotSenku.Model.Builders;
using dotSenku.Model.EndingRules;
using dotSenku.Model.CountingRules;
using dotSenku.Model.Movements;
using dotSenku.Model;

namespace dotSenku.Model.Builders
{
	/// <summary>
	/// Builds a Dudeney variant game
	/// </summary>
	public class DudeneyBuilder : GameBuilder
	{
		public DudeneyBuilder()
		{			
		}

		/// <summary>
		/// builds a cross board
		/// </summary>
		/// <returns></returns>
		override protected Board buildBoard()
		{	
			SquareBoardBuilder boardBuilder = new SquareBoardBuilder();
			return boardBuilder.build(5, 5, 3, 3);
		}

		/// <summary>
		/// Dudeney uses the peg movement counting rule
		/// </summary>
		/// <returns></returns>		
		override protected CountingRule buildCountingRule()			
		{
			return new PegMovementCounter();
		}
		
		/// <summary>
		/// Dudeney uses standard ending rule
		/// </summary>
		/// <param name="aBoard"></param>
		/// <returns></returns>
		override protected EndingRule buildEndingRule(Board aBoard)							
		{
			return new StandardEndingRule(aBoard);
		}
		
		/// <summary>
		/// builds the jump and diagonal jump movement factories
		/// </summary>
		/// <returns></returns>
		override protected MovementRule buildMovementRule()
		{					
			ArrayList factories = new ArrayList();
			factories.Add( new JumpFactory() );
			factories.Add( new DiagonalJumpFactory() );		
			return new MovementRule(factories);
		}
		

	}
}
