using System;
using dotSenku.Model;
using dotSenku.Model.CountingRules;
using dotSenku.Model.EndingRules;
using dotSenku.Model.Movements;


namespace dotSenku.Model.Builders
{
	/// <summary>
	/// Builds a Game.
	/// </summary>
	public abstract class GameBuilder : MarshalByRefObject
	{
		public GameBuilder()
		{			
		}
		
		/// <summary>
		/// builds the board for a specific game
		/// </summary>
		protected abstract Board buildBoard();
			
		/// <summary>
		/// builds the counting rule for a specific game
		/// </summary>
		protected abstract CountingRule buildCountingRule();
			
		/// <summary>
		/// builds the ending rule for a specific game
		/// </summary>
		/// <param name="aBoard"></param>
		protected abstract EndingRule buildEndingRule(Board aBoard);
							
				
		/// <summary>
		/// builds the movement rule for a specific game
		/// </summary>
		protected abstract MovementRule buildMovementRule();
			
		/// <summary>
		/// Builds the game
		/// </summary>
		/// <returns></returns>		
		public Game build()
		{					
			Game aGame = new Game();
			aGame.movementRule = this.buildMovementRule();
			aGame.board = this.buildBoard();
			aGame.endingRule = this.buildEndingRule(aGame.board);
			aGame.countingRule = this.buildCountingRule();
			Console.WriteLine("estoy construyendo remotamente el juego");
			return aGame;
		}									  

	}
}
