using System;
using System.Collections;

namespace dotSenku.Model
{
	/// <summary>
	/// Represents a Matrix
	/// </summary>
	public class Matrix : MarshalByRefObject
	{

		#region properties
		/// <summary>
		/// Height of the matrix
		/// </summary>
		private int _height;
		public int height
		{
			get { return this._height; }
			set { this._height = value; }
		}

		/// <summary>
		/// Width
		/// </summary>
		private int _width;
		public int width
		{
			get { return this._width; }
			set { this._width = value; }
		}

		private object[][] _positions; 
		#endregion

		public Matrix()
		{			
		}

		public Matrix(int aWidth, int aHeight)
		{			
			this.width = aWidth;
			this.height = aHeight;
			this._positions = new object[aWidth][];
			for(int i=0;i<aWidth;i++)
			{
				this._positions[i] = new object[aHeight];
			}
		}

		/// <summary>
		/// Returns a collection with the positions of the matrix that are not empty
		/// </summary>
		/// <returns></returns>
		public ICollection noNilPositions()
		{
			ArrayList notEmptyPositions = new ArrayList();
			for(int i = 0; i < this.width; i++)
				for(int j = 0; j < this.height; j++)
					if (this._positions[i][j] != null)
						notEmptyPositions.Add(this._positions[i][j]);
			return notEmptyPositions;
		}

		/// <summary>
		/// Return the object located at the specified position
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <returns></returns>
		public object at(int x, int y)
		{
			return this._positions[x][ y];
		}

		/// <summary>
		/// Set the object at the specified position
		/// </summary>
		/// <param name="x"></param>
		/// <param name="y"></param>
		/// <returns></returns>
		public void atPut(int x, int y, object value)
		{
			this._positions[x][ y] = value;
		}


		/// <summary>
		/// Return the position of the given object in the matrix
		/// </summary>
		/// <returns></returns>
		public Point positionOf(object anObject)
		{
			for(int i = 0; i < this.width; i++)
				for(int j = 0; j < this.height; j++)
					if (this._positions[i][ j] == anObject)
						return new Point(i, j);
			
			return null;
		}

		/// <summary>
		/// return a collection with the asked positions
		/// </summary>
		/// <param name="aPoint"></param>
		/// <param name="aWidth"></param>
		/// <param name="anHeight"></param>
		/// <returns></returns>
		public ICollection rowsAndColumnsFrom(Point aPoint, int aWidth, int anHeight)
		{	
			ArrayList positions = new ArrayList();
			for(int i = aPoint.x; i < aPoint.x + this.width - 1; i++)
				for(int j = aPoint.y; j < aPoint.y + this.height - 1; j++)
					if (this._positions[i][ j] != null)
						positions.Add(this._positions[i][ j]);
			return positions;				
		}
	}
}
