using System;
using System.Collections;
using dotSenku.Model.Builders;
using dotSenku.Model.EndingRules;
using dotSenku.Model.CountingRules;
using dotSenku.Model.Movements;
using dotSenku.Model;
using setPoint.programAnnotation;

namespace dotSenku.Model.Builders
{
	/// <summary>
	/// Builds a Senku variant game
	/// </summary>
	public class SenkuBuilder : GameBuilder
	{
		[ProgramAnnotation("semantics://perspectives/CTS#constructor")]
		public SenkuBuilder()
		{			
		}

		/// <summary>
		/// builds a cross board
		/// </summary>
		/// <returns></returns>
		override protected Board buildBoard()
		{				
			CrossBoardBuilder boardBuilder = new CrossBoardBuilder();
			return boardBuilder.build(7, 7, 3, 3);
		}

		/// <summary>
		/// Senku uses the single movement counting rule
		/// </summary>
		/// <returns></returns>		
		override protected CountingRule buildCountingRule()			
		{
			return new SingleMovementCounter();
		}
		
		/// <summary>
		/// Senku uses standard ending rule
		/// </summary>
		/// <param name="aBoard"></param>
		/// <returns></returns>
		override protected EndingRule buildEndingRule(Board aBoard)							
		{
			return new StandardEndingRule(aBoard);
		}
		
		/// <summary>
		/// builds the jump movement factories
		/// </summary>
		/// <returns></returns>
		override protected MovementRule buildMovementRule()
		{					
			ArrayList factories = new ArrayList();
			factories.Add( new JumpFactory() );			
			return new MovementRule(factories);
		}
		
	}
}
