using System;
using System.Collections;
using dotSenku.Model;

namespace dotSenku.Model.EndingRules
{
	/// <summary>
	/// The game finishes when no peg can be moved or when there is only one left
	/// </summary>
	public class StandardEndingRule : EndingRule
	{
		public StandardEndingRule()
		{
		}

		/// <summary>
		/// creates the ending rule for a specific board
		/// </summary>
		/// <param name="aBoard"></param>
		public StandardEndingRule(Board aBoard)
		{
			this.endingSquare = (Square) aBoard.matrix.at( aBoard.matrix.width / 2, aBoard.matrix.height / 2);
		}
								   

		/// <summary>
		/// square where you have to leave the last peg if you want to win the game
		/// </summary>
		private Square _endingSquare;
		public Square endingSquare
		{
			get { return this._endingSquare; }
			set { this._endingSquare = value; }
		}
		
				
		/// <summary>
		/// finishes the game when there are no more posible movements or
		/// when only one peg remains on the board"
		/// </summary>
		/// <param name="aGame"></param>
		override public void analyzeEndingFor(Game aGame)
		{					
			ICollection aSquareCollection = aGame.board.squares;
			ArrayList aSetSquareCollection = new ArrayList();			
			foreach(Square aSquare in aSquareCollection)				
				if (aSquare.isSet) 
					aSetSquareCollection.Add(aSquare);				
						
			if (aSetSquareCollection.Count == 1)			
				this.determineIfWon((Square) aSetSquareCollection[0], aGame);			
			else
				this.determineIfLost(aSquareCollection, aGame);			
		}
		
		
		/// <summary>
		/// decides whether the player has lost the game or he may continue playing
		/// </summary>
		/// <param name="aSquareCollection"></param>
		/// <param name="aGame"></param>
		private void determineIfLost(ICollection aSquareCollection, Game aGame)													 
		{								
			foreach(Square aSquare in aSquareCollection)
				if (aGame.potentialMovementsFor(aSquare).Count > 0)
					return;			
			aGame.lost();				
		}
		
		/// <summary>
		/// decides whether the player has won the game when there's only one peg on the board
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aGame"></param>
		private void determineIfWon(Square aSquare, Game aGame)										  
		{			
			if (aSquare == this.endingSquare)
				aGame.won();
			else
				aGame.lost();				
		}
	}
}
