using System;
using System.Collections;
using dotSenku.Model.CountingRules;
using dotSenku.Model.EndingRules;
using dotSenku.Model.Movements;

namespace dotSenku.Model
{
	[Serializable]
	public delegate void CollAsParam(ICollection aCollection);

	/// <summary>
	/// Senku game
	/// </summary>
	public class Game: MarshalByRefObject
	{
		public Game()
		{			
		}

		#region Game components

		private Board _board;
		/// <summary>
		/// Game board
		/// </summary>
		public Board board
		{
			get { return this._board; }
			set { this._board = value; }
		}

		private CountingRule _countingRule;
		/// <summary>
		/// rule that determines what is considered a new movement
		/// </summary>
		public CountingRule countingRule
		{
			get { return this._countingRule; }
			set { this._countingRule = value; }
		}

		
		private EndingRule _endingRule;
		/// <summary>
		/// rule that determines when the game is finished
		/// </summary>
		public EndingRule endingRule
		{
			get { return this._endingRule; }
			set { this._endingRule = value; }
		}

		private MovementRule _movementRule;
		/// <summary>
		/// rule that determines whiche are the possible movement for this game
		/// </summary>
		public MovementRule movementRule
		{
			get { return this._movementRule; }
			set { this._movementRule = value; }
		}
		

		#endregion

		#region event triggers

		public event NoParams onLost;
		/// <summary>
		/// the player has lost the game
		/// </summary>		
		public void lost() {
			if (this.onLost != null)
				this.onLost();
		}		

		public event NoParams onWon;
		/// <summary>
		/// the player has won the game
		/// </summary>		
		public void won() {
			if (this.onWon != null)
				this.onWon();
		}		
		
		public event CollAsParam onPotentialMovementsRequired;		
		/// <summary>
		/// the player wants to move!
		/// </summary>
		/// <param name="aSquare"></param>
		public void playerWantsToMoveFrom(Square aSquare)
		{	
			if (this.onPotentialMovementsRequired != null)
				this.onPotentialMovementsRequired(this.potentialMovementsFor(aSquare));
		}
		#endregion

		
		#region evaluating
		/// <summary>
		/// executes the specified movement
		/// </summary>
		/// <param name="aMovement"></param>
		public void execute(Movement aMovement)
		{	
			aMovement.execute();
			this.countingRule.count(aMovement);
			this.endingRule.analyzeEndingFor(this);
		}

		/// <summary>
		/// returns a collection of the movements that can be executed 
		/// in the current situation of the board starting from the specified square
		/// </summary>
		/// <returns></returns>
		public ICollection potentialMovementsFor(Square aSquare)
		{
			return this.movementRule.potentialMovementsFor(aSquare, this.board);			 
		}

		#endregion
		
	}
}
