using System;
using System.Collections;
using dotSenku.Model;

namespace dotSenku.Model.Movements
{
	/// <summary>
	/// DiagonalJumpFactory.
	/// </summary>
	public class DiagonalJumpFactory : MovementFactory
	{
		public DiagonalJumpFactory()
		{
		}

		#region auxiliar methods

		/// <summary>
		/// adds to the collection the diagonal jump movement to the above, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)
		{															
			if (aBoard.hasAboveLeft(aSquare))
			{
				Square aboveLeftSquare = aBoard.aboveLeft(aSquare);
				if (aBoard.hasAboveLeft(aboveLeftSquare))				
					this.addIfValidFrom(aSquare, aboveLeftSquare, aBoard.aboveLeft(aboveLeftSquare), aCollection);
					
			}
		}
			
		/// <summary>
		/// adds to the collection the diagonal jump movement to the above, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)
		{															
			if (aBoard.hasAboveRight(aSquare))
			{
				Square aboveRightSquare = aBoard.aboveRight(aSquare);
				if (aBoard.hasAboveRight(aboveRightSquare))				
					this.addIfValidFrom(aSquare, aboveRightSquare, aBoard.aboveRight(aboveRightSquare), aCollection);
					
			}
		}

		/// <summary>
		/// adds to the collection the diagonal jump movement to the under, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)
		{															
			if (aBoard.hasUnderLeft(aSquare))
			{
				Square underLeftSquare = aBoard.underLeft(aSquare);
				if (aBoard.hasUnderLeft(underLeftSquare))				
					this.addIfValidFrom(aSquare, underLeftSquare, aBoard.underLeft(underLeftSquare), aCollection);
					
			}
		}
			
		/// <summary>
		/// adds to the collection the diagonal jump movement to the under, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)
		{															
			if (aBoard.hasUnderRight(aSquare))
			{
				Square underRightSquare = aBoard.underRight(aSquare);
				if (aBoard.hasUnderRight(underRightSquare))				
					this.addIfValidFrom(aSquare, underRightSquare, aBoard.underRight(underRightSquare), aCollection);
					
			}
		}
			
				
		/// <summary>
		/// adds to the collection the jump movement, if it's possible to execute it
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <param name="aCollection"></param>		
		public void addIfValidFrom(Square aSourceSquare, Square aMiddleSquare, Square aTargetSquare, ArrayList aCollection)
		{
			if (this.isValidFrom(aSourceSquare, aMiddleSquare, aTargetSquare))			
				aCollection.Add(new JumpMovement(aSourceSquare, aMiddleSquare, aTargetSquare));			
		}
		
		/// <summary>
		/// verifies dynamic conditions of a jump
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <returns></returns>						
		public bool isValidFrom(Square aSourceSquare, Square aMiddleSquare, Square aTargetSquare)
		{
			return ((!aTargetSquare.isSet)&&(aSourceSquare.isSet)&&(aMiddleSquare.isSet));
		}		

		#endregion

		/// <summary>
		/// returns the posible diagonal jump movements that can be executed on this board 
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <returns></returns>
		override public ICollection potentialMovementsFor(Square aSquare, Board aBoard)
		{											 		
			ArrayList movs = new ArrayList();
			this.addAboveLeftMovementIfPossibleFrom(aSquare, aBoard, movs); 
			this.addAboveRightMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addUnderLeftMovementIfPossibleFrom(aSquare, aBoard, movs); 
			this.addUnderRightMovementIfPossibleFrom(aSquare, aBoard, movs);
			return movs;			
		}
	}
}
