using System;
using System.Collections;
using dotSenku.Model;

namespace dotSenku.Model.Movements
{
	/// <summary>
	/// JumpFactory.
	/// </summary>
	public class JumpFactory : MovementFactory
	{
		public JumpFactory()
		{		
		}

		#region auxiliar methods

		/// <summary>
		/// adds to the collection the jump movement to the above, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasAbove(aSquare))
			{
				Square aboveSquare = aBoard.above(aSquare);
				if (aBoard.hasAbove(aboveSquare))
					this.addIfValidFrom(aSquare, aboveSquare, aBoard.above(aboveSquare), aCollection);
			}
		}

		/// <summary>
		/// adds to the collection the jump movement to the under, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasUnder(aSquare))
			{
				Square underSquare = aBoard.under(aSquare);
				if (aBoard.hasUnder(underSquare))
					this.addIfValidFrom(aSquare, underSquare, aBoard.under(underSquare), aCollection);
			}
		}

		/// <summary>
		/// adds to the collection the jump movement to the left, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasLeft(aSquare))
			{
				Square leftSquare = aBoard.left(aSquare);
				if (aBoard.hasLeft(leftSquare))
					this.addIfValidFrom(aSquare, leftSquare, aBoard.left(leftSquare), aCollection);
			}
		}

		/// <summary>
		/// adds to the collection the jump movement to the right, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasRight(aSquare))
			{
				Square rightSquare = aBoard.right(aSquare);
				if (aBoard.hasRight(rightSquare))
					this.addIfValidFrom(aSquare, rightSquare, aBoard.right(rightSquare), aCollection);
			}
		}
		
		/// <summary>
		/// adds to the collection the jump movement, if it's possible to execute it
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <param name="aCollection"></param>
		private void addIfValidFrom(Square aSourceSquare, Square aMiddleSquare, Square aTargetSquare, ArrayList aCollection)																				
		{
			if (this.isValidFrom(aSourceSquare, aMiddleSquare, aTargetSquare))
				aCollection.Add(new JumpMovement(aSourceSquare, aMiddleSquare, aTargetSquare));
		}


		/// <summary>
		/// verifies de dynamic conditions of a jump 
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <returns></returns>
		private bool isValidFrom(Square aSourceSquare, Square aMiddleSquare, Square aTargetSquare)
		{
			return ((!aTargetSquare.isSet)&&(aSourceSquare.isSet)&&(aMiddleSquare.isSet));
		}

		#endregion

		/// <summary>
		/// returns the posible jump movements that can be executed on this board 
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <returns></returns>
		override public ICollection potentialMovementsFor(Square aSquare, Board aBoard)
		{											 					
			ArrayList movs = new ArrayList();
			this.addAboveMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addUnderMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addLeftMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addRightMovementIfPossibleFrom(aSquare, aBoard, movs);
			return movs;
		}

	}
}
