using System;
using System.Collections;
using dotSenku.Model;

namespace dotSenku.Model.Movements
{
	/// <summary>
	/// StepFactory.
	/// </summary>
	public class StepFactory : MovementFactory
	{
		public StepFactory()
		{		
		}

		#region auxiliar methods

		/// <summary>
		/// adds to the collection the step movement to the above, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasAbove(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.above(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the under, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasUnder(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.under(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the left, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasLeft(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.left(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the right, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasRight(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.right(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the aboveLeft, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasAboveLeft(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.aboveLeft(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the aboveRight, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addAboveRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasAboveRight(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.aboveRight(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the underLeft, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderLeftMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasUnderLeft(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.underLeft(aSquare), aCollection);			
		}

		/// <summary>
		/// adds to the collection the step movement to the underRight, if it's possible to execute it
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <param name="aCollection"></param>
		private void addUnderRightMovementIfPossibleFrom(Square aSquare, Board aBoard, ArrayList aCollection)																
		{
			if (aBoard.hasUnderRight(aSquare))			
				this.addIfValidFrom(aSquare, aBoard.underRight(aSquare), aCollection);			
		}

		
		/// <summary>
		/// adds to the collection the step movement, if it's possible to execute it
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <param name="aCollection"></param>
		private void addIfValidFrom(Square aSourceSquare, Square aTargetSquare, ArrayList aCollection)																				
		{
			if (this.isValidFrom(aSourceSquare, aTargetSquare))
				aCollection.Add(new StepMovement(aSourceSquare, aTargetSquare));
		}


		/// <summary>
		/// verifies de dynamic conditions of a step 
		/// </summary>
		/// <param name="aSourceSquare"></param>
		/// <param name="aMiddleSquare"></param>
		/// <param name="aTargetSquare"></param>
		/// <returns></returns>
		private bool isValidFrom(Square aSourceSquare, Square aTargetSquare)
		{
			return ((!aTargetSquare.isSet)&&(aSourceSquare.isSet));
		}

		#endregion

		/// <summary>
		/// returns the posible step movements that can be executed on this board 
		/// </summary>
		/// <param name="aSquare"></param>
		/// <param name="aBoard"></param>
		/// <returns></returns>
		override public ICollection potentialMovementsFor(Square aSquare, Board aBoard)
		{											 		
			ArrayList movs = new ArrayList();
			this.addAboveMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addUnderMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addLeftMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addRightMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addAboveLeftMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addAboveRightMovementIfPossibleFrom(aSquare, aBoard, movs);
			this.addUnderLeftMovementIfPossibleFrom(aSquare, aBoard, movs);			
			this.addUnderRightMovementIfPossibleFrom(aSquare, aBoard, movs);			
			return movs;
		}

	}
}
