using System.Collections;

namespace dotSenku.View {
	/// <summary>
	/// Summary description for BoardMorph.
	/// </summary>
	public class BoardMorph : System.Windows.Forms.UserControl {
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public BoardMorph(dotSenku.View.frmPlayRoom aPlayRoom, dotSenku.Model.Board aBoard) {
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			this.board = aBoard;
			this.createSquaresOn(aPlayRoom, aBoard);
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null)
					components.Dispose();
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// BoardMorph
			// 
			this.BackColor = System.Drawing.Color.Black;
			this.Name = "BoardMorph";
			this.Size = new System.Drawing.Size(250, 250);
			this.Load += new System.EventHandler(this.BoardMorph_Load);

		}

		#endregion

		private void BoardMorph_Load(object sender, System.EventArgs e) {}

		/// <summary>
		/// show the given movement collection on the board
		/// </summary>
		/// <param name="aMovementCollection"></param>
		public void createPotentialMovementsOn(dotSenku.View.frmPlayRoom aPlayRoom, ICollection aMovementCollection) {
			foreach (dotSenku.Model.Movements.Movement aMovement in aMovementCollection) {
				MovementMorph movement = new MovementMorph(aPlayRoom, this, aMovement);
				this.Controls.Add(movement);
				movement.BringToFront();
				this.movements.Add(movement);
			}
		}

		public void abandonMovements() {
			foreach (MovementMorph movement in this.movements)
				movement.Dispose();
		}

		#region accessing

		private SortedList _squares = new SortedList();

		public SortedList squares {
			get { return this._squares; }
		}

		private ArrayList _movements = new ArrayList();

		public ArrayList movements {
			get { return this._movements; }
		}

		private dotSenku.Model.Board _board;

		public dotSenku.Model.Board board {
			get { return this._board; }
			set { this._board = value; }
		}

		private bool _enabled;

		public bool enabled {
			get { return this._enabled; }
			set {
				this._enabled = value;
				foreach (SquareMorph square in this.squares.Values)
					square.enabled = value;
			}
		}

		/// <summary>
		/// obtains SquareMorph corresponding to model's aSquare
		/// </summary>
		/// <param name="aSquare"></param>
		public VisibleSquareMorph squareMorphOf(dotSenku.Model.Square aSquare) {
			foreach (SquareMorph squareMorph in this.squares.Values)
				if (squareMorph is VisibleSquareMorph)
					if (((VisibleSquareMorph) squareMorph).square == aSquare)
						return (VisibleSquareMorph) squareMorph;
			return null;
		}

		#endregion

		#region children creation

		private void createSquaresOn(dotSenku.View.frmPlayRoom aPlayRoom, dotSenku.Model.Board aBoard) {
			int width = (this.Width - (aBoard.matrix.width + 1)*3)/aBoard.matrix.width;
			int height = (this.Height - (aBoard.matrix.height + 1)*3)/aBoard.matrix.height;
			for (int i = 0; i < aBoard.matrix.width; i++) {
				int left = 4 + (3 + width)*i;
				for (int j = 0; j < aBoard.matrix.height; j++) {
					int top = 4 + (3 + height)*j;
					SquareMorph square = null;
					if (aBoard.matrix.at(i, j) == null)
						square = new InvisibleSquareMorph(left, top, width, height);
					else
						square = new VisibleSquareMorph(aPlayRoom, (dotSenku.Model.Square) board.matrix.at(i, j), left, top, width, height);
					this.squares.Add(square, square);
					this.Controls.Add(square);
				}
			}
		}

		#endregion
	}
}