using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace dotSenku.View {
	/// <summary>
	/// Summary description for MovementMorph.
	/// </summary>
	public class MovementMorph : System.Windows.Forms.UserControl {
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private Point[] _coordinates = new Point[4];

		public MovementMorph(frmPlayRoom aPlayRoom, BoardMorph aBoard, dotSenku.Model.Movements.Movement aMovement) {
			this.movement = aMovement;
			this.playRoom = aPlayRoom;
			VisibleSquareMorph aSourceSquare = aBoard.squareMorphOf(aMovement.source);
			VisibleSquareMorph aTargetSquare = aBoard.squareMorphOf(aMovement.target);
			Point sourceCenter = new Point(aSourceSquare.Left + aSourceSquare.Width/2, aSourceSquare.Top + aSourceSquare.Height/2);
			Point targetCenter = new Point(aTargetSquare.Left + aTargetSquare.Width/2, aTargetSquare.Top + aTargetSquare.Height/2);
			int addY1 = 0;
			int addY2 = 0;
			int addX1 = -4;
			int addX2 = 4;
			if (sourceCenter.Y == targetCenter.Y) {
				addY1 = -4;
				addY2 = 4;
				addX1 = 0;
				addX2 = 0;
			}

			this._coordinates[0] = new Point(sourceCenter.X + addX1, sourceCenter.Y + addY1);
			this._coordinates[1] = new Point(targetCenter.X + addX1, targetCenter.Y + addY1);
			this._coordinates[2] = new Point(targetCenter.X + addX2, targetCenter.Y + addY2);
			this._coordinates[3] = new Point(sourceCenter.X + addX2, sourceCenter.Y + addY2);

			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();
			// TODO: Add any initialization after the InitializeComponent call

		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// MovementMorph
			// 
			this.BackColor = System.Drawing.Color.SandyBrown;
			this.Cursor = System.Windows.Forms.Cursors.Hand;
			this.Name = "MovementMorph";
			this.Size = new System.Drawing.Size(250, 250);
			this.Click += new System.EventHandler(this.MovementMorph_Click);
			this.MouseEnter += new System.EventHandler(this.MovementMorph_MouseEnter);
			this.MouseLeave += new System.EventHandler(this.MovementMorph_MouseLeave);

		}

		#endregion

		private void MovementMorph_MouseEnter(object sender, System.EventArgs e) {
			this.BackColor = System.Drawing.Color.LightGreen;
		}

		private void MovementMorph_MouseLeave(object sender, System.EventArgs e) {
			this.BackColor = System.Drawing.Color.SandyBrown;
		}

		/// <summary>
		///  triggers the execution of contained movement
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MovementMorph_Click(object sender, EventArgs e) {
			this.playRoom.execute(this.movement);
			this.playRoom.board.abandonMovements();
		}

		override protected void OnPaint(PaintEventArgs e) {
			// Create a path that is made up of a single polygon.			
			GraphicsPath path = new GraphicsPath();
			path.AddPolygon(this._coordinates);

			// Construct a region that is based on the path.
			Region region = new Region(path);
			// Draw the outline of the region.
			Pen pen = new Pen(Color.SandyBrown, 0);
			Graphics graphics = e.Graphics;
			graphics.DrawPath(pen, path);
			// Set the clipping region of the Graphics object.			
			graphics.SetClip(region, CombineMode.Replace);
			this.Region = region;
		}

		#region properties

		private frmPlayRoom _playRoom;

		public frmPlayRoom playRoom {
			get { return this._playRoom; }
			set { this._playRoom = value; }
		}

		private dotSenku.Model.Movements.Movement _movement;

		public dotSenku.Model.Movements.Movement movement {
			get { return this._movement; }
			set { this._movement = value; }
		}

		#endregion
	}
}