using System.Windows.Forms;

namespace dotSenku.View {
	/// <summary>
	/// Summary description for VisibleSquareMorph.
	/// </summary>
	public class VisibleSquareMorph : dotSenku.View.SquareMorph {
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public VisibleSquareMorph(dotSenku.View.frmPlayRoom aPlayRoom, dotSenku.Model.Square aSquare, int left, int top, int width, int height) : base(left, top, width, height) {
			InitializeComponent();
			this.playRoom = aPlayRoom;
			this.square = aSquare;
			this.square.onSquareFreed += new dotSenku.Model.NoParams(this.free);
			this.square.onSquareSet += new dotSenku.Model.NoParams(this.set);
			if (this.isSet)
				this.createPeg();
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		#region Component Designer generated code

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// VisibleSquareMorph
			// 
			this.BackColor = System.Drawing.Color.LightSkyBlue;
			this.Name = "VisibleSquareMorph";
		}

		#endregion

		#region properties		

		private dotSenku.View.frmPlayRoom _playRoom;

		public dotSenku.View.frmPlayRoom playRoom {
			get { return this._playRoom; }
			set { this._playRoom = value; }
		}

		private dotSenku.Model.Square _square;

		public dotSenku.Model.Square square {
			get { return this._square; }
			set { this._square = value; }
		}

		private dotSenku.View.PegMorph _peg;

		private dotSenku.View.PegMorph peg {
			get { return this._peg; }
			set { this._peg = value; }
		}

		override public bool enabled {
			set {
				base.enabled = value;
				if (this.peg != null)
					this.peg.enabled = value;
			}
		}

		/// <summary>
		/// is there a Peg in the square?
		/// </summary>
		public bool isSet {
			get { return this.square.isSet; }
		}

		#endregion

		#region initialization

		/// <summary>
		/// creates contained peg
		/// </summary>		
		public void createPeg() 
		{
			if (this.InvokeRequired)
				this.BeginInvoke(new MethodInvoker(this.createPegInThread)) ;
			else
				this.createPegInThread();			
		}

		private void createPegInThread() 
		{
			this.peg = new PegMorph(this.GetType().GetMethod("playerWantsToMove"), this);			
			this.Controls.Add(this.peg);
			this.peg.Top = (this.Height - this.peg.Height)/2;
			this.peg.Left = (this.Width - this.peg.Width)/2;
			this.peg.BringToFront();						
		}

		#endregion

		#region Events-processing		

		/// <summary>
		/// corresponding square in model has been set
		/// </summary>
		public void set() {
			this.createPeg();
		}

		/// <summary>
		/// corresponding square in model has been freed
		/// </summary>		
		public void free() 
		{						
			if (this.InvokeRequired)
				this.BeginInvoke(new MethodInvoker(this.freeInThread)) ;
			else
				this.freeInThread();			
		}

		private void freeInThread()
		{
			this.Controls.RemoveAt(0);
			//this.peg.Dispose();
			this.peg = null;				
		}

		/// <summary>
		/// the potential movements for current square have been asked for
		/// </summary>
		public void playerWantsToMove() {
			this.playRoom.playerWantsToMoveFrom(this);
		}

		#endregion
	}
}